/*
  Automatisierte Stoppuhren zur Bestimmmung der Schallgeschwindigkeiten mit Schallauslöser

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/



p5.disableFriendlyErrors = true; // disables FES

// LAYOUT
let mn;
let button_width;
let button_height;
let button_y;
let button_rad;
let button_rad2;
let text_size
let line_width;
let button_round;

let button_l_pressed;
let button_m_pressed;

const mode_audio_init = 0;
const mode_auto_start = 1;
const mode_auto_stopp = 2;
const mode_auto_start_stopp = 3;

const button_l_c1 = [75, 252, 128];
const button_r_c1 = [75, 252, 217];
const button_l_c2 = [244, 12, 43];
const button_r_c2 = [244, 97, 12];
const button_r_c3 = [255, 255, 255];
const button_m_c1 = [240, 240, 240];
const button_m_c2 = [240, 240, 240];
const button_m_c3 = [240, 240, 240];

let mouse_x_min_l;
let mouse_x_max_l;
let mouse_y_min_l;
let mouse_y_max_l;

let mouse_x_m;
let mouse_y_m;

let layout_line1_start_x;
let layout_line1_end_x;
let layout_line1_y;
let layout_line2_start_x;
let layout_line2_end_x;
let layout_line2_y;
let layout_line3_start_x;
let layout_line3_end_x;
let layout_line3_y;
let layout_line4_start_x;
let layout_line4_end_x;
let layout_line4_y;


// GLOBAL
let mode;
let stopwatch_l_running;
let stopwatch_r_running;
let auto_pause;

let start_time;
let end_time_l;
let end_time_r;

let inp;
let inp_dist;
let inp_auto;

let mic;
let threshold;
let max_rad;
let threshold_selected;
let vol_low;




function setup() {
  createCanvas(windowWidth, windowHeight);

  mic = new p5.AudioIn();
  mic.start();

  inp = createInput('100');
  inp.input(myInputEvent);
  inp_dist = 100;

  inp_auto = createInput('0.00');
  inp_auto.input(myInputEventAuto);

  calculateLayout();

  mode = 0;
  threshold = 0.5;
  stopwatch_l_running = false;
  stopwatch_r_running = false;
  start_time = 0;
  end_time_l = 0;
  end_time_r = 0;

  vol_low = false;
}


function draw() {
  background(255);

  if (stopwatch_l_running)
    end_time_l = (millis() - start_time) * 0.001;

  if (stopwatch_r_running)
    end_time_r = (millis() - start_time) * 0.001;

  renderLayout();
  renderButtons();
}


function calculateLayout() {
  mn = width * height * 0.000014;
  text_size = height * 0.07;
  line_width = mn * 0.4;
  button_width = (width - 4 * mn) * 0.5;
  button_height = height - 6 * mn - 3 * text_size;
  button_y = 2 * mn + text_size;
  button_rad = text_size + mn;
  button_round = 4 * mn;

  mouse_x_min_l = mn;
  mouse_x_max_l = mn + button_width;
  mouse_y_min_l = button_y;
  mouse_y_max_l = button_y + button_height;

  mouse_x_m = width * 0.5;
  mouse_y_m = button_y + button_height * 0.5;
  button_rad2 = button_rad * button_rad;

  layout_line1_start_x = mn;
  layout_line1_end_x = button_width - mn * 2;
  layout_line1_y = mn * 4 + text_size * 2 + button_height;
  layout_line2_start_x = width * 0.5 - mn * 2;
  layout_line2_end_x = width * 0.5 + mn * 2;
  layout_line2_y = mn * 3 + text_size * 2 + button_height;
  layout_line3_start_x = layout_line2_start_x;
  layout_line3_end_x = layout_line2_end_x;
  layout_line3_y = mn * 4 + text_size * 2 + button_height;
  layout_line4_start_x = width - text_size - mn;
  layout_line4_end_x = width - mn;
  layout_line4_y = layout_line1_y;

  button_l_pressed = false;
  button_m_pressed = false;

  threshold_selected = false;
  auto_pause = true;

  stroke(0);
  strokeWeight(line_width);
  strokeCap(ROUND);

  fill(255);

  textSize(text_size);
  textAlign(CENTER);
  textFont('Avenir')
  textStyle(NORMAL);

  inp.position(mn * 2, mn * 3 + text_size + button_height);
  inp.size(button_width - mn * 7 - text_size * 1.5, text_size * 0.7);
  inp.style('text-align: center');
  inp.style('font-size: ' + str(text_size * 0.9) + 'px');
  inp.style('font-weight: bold');

  inp_auto.position(mn * 2, mn);
  inp_auto.size(button_width - mn * 2 - text_size * 1.5, text_size * 0.7);
  inp_auto.style('text-align: center');
  inp_auto.style('font-size: ' + str(text_size * 0.9) + 'px');
  inp_auto.style('font-weight: bold');

  if (button_height > button_width) {
    max_rad = button_width;
  } else {
    max_rad = button_height;
  }
}


function renderLayout() {
  push();
  line(layout_line1_start_x, layout_line1_y, layout_line1_end_x, layout_line1_y);

  line(layout_line2_start_x, layout_line2_y, layout_line2_end_x, layout_line2_y);
  line(layout_line3_start_x, layout_line3_y, layout_line3_end_x, layout_line3_y);

  line(layout_line4_start_x, layout_line4_y, layout_line4_end_x, layout_line4_y);

  fill(0);
  strokeWeight(line_width * 0.5);
  if (mode == 2)
    text(end_time_l.toFixed(2), mn + button_width * 0.5, mn + text_size);

  text(end_time_r.toFixed(2), (width + button_width) * 0.5 + mn, mn + text_size);

  let delta_time = abs(end_time_l - end_time_r);
  text((inp_dist / delta_time).toFixed(2), (width + button_width - text_size) * 0.5 + mn * 2, mn * 4 + text_size * 2.25 + button_height);
  textSize(text_size * 0.9);
  text(delta_time.toFixed(2), button_width * 0.5 - mn * 3, mn * 5 + text_size * 2.75 + button_height);
  textSize(text_size);
  text("s", button_width - mn * 5, mn * 5 + text_size * 2.75 + button_height);
  text("m", button_width - mn * 5, mn * 3 + text_size * 1.75 + button_height);

  text("m", width - text_size * 0.5 - mn, mn * 3 + text_size * 2 + button_height);
  text("s", width - text_size * 0.5 - mn, mn * 5 + text_size * 2.5 + button_height);

  textAlign(RIGHT);
  text("s", button_width, mn + text_size);
  text("s", width - mn * 2, mn + text_size);

  pop();
}


function checkButtons() {
  let dist_temp = distSquared(mouseX, mouseY, mouse_x_m, mouse_y_m);
  if (dist_temp < button_rad2) {
    button_m_pressed = true;
    switchMode();
  }

  switch (mode) {
    case 0:
      if (mouseX > mouse_x_min_l && mouseX < mouse_x_max_l &&
        mouseY > mouse_y_min_l && mouseY < mouse_y_max_l &&
        dist_temp > button_rad2 && !button_l_pressed) {

        button_l_pressed = true;
        switchMode();
      }
      break;

    case 1:
      if (mouseX > mouse_x_min_l && mouseX < mouse_x_max_l &&
        mouseY > mouse_y_min_l && mouseY < mouse_y_max_l &&
        dist_temp > button_rad2) {

        button_l_pressed = true;
        if (stopwatch_r_running)
          stopwatch_r_running = false;
      }
      break;

    case 2:
      if (mouseX > mouse_x_min_l && mouseX < mouse_x_max_l &&
        mouseY > mouse_y_min_l && mouseY < mouse_y_max_l &&
        dist_temp > button_rad2) {

        button_l_pressed = true;
        if (!stopwatch_l_running && !stopwatch_r_running) {
          start_time = millis();
          end_time_l = start_time;
          end_time_r = start_time;
          stopwatch_l_running = true;
          stopwatch_r_running = true;
        } else if (stopwatch_l_running)
          stopwatch_l_running = false;
      }
      break;

    case 3:
      if (mouseX > mouse_x_min_l && mouseX < mouse_x_max_l &&
        mouseY > mouse_y_min_l && mouseY < mouse_y_max_l &&
        dist_temp > button_rad2) {

        button_l_pressed = true;
        auto_pause ^= true;
      }
      break;
  }
}


function renderButtons() {
  push();
  textAlign(CENTER)
  let button_l_c;
  let button_r_c;
  let button_m_c;

  if (stopwatch_l_running)
    button_l_c = color(button_l_c2);
  else
    button_l_c = color(button_l_c1);

  if (stopwatch_r_running)
    button_r_c = color(button_r_c2);
  else
    button_r_c = color(button_r_c1);

  switch (mode) {
    case 0:
      inp_auto.hide();

      push();
      button_r_c = color(button_r_c3);
      fill(button_r_c);
      rect(width * 0.5 + mn, button_y, button_width, button_height, button_round);

      if (button_l_pressed) {
        button_l_c.setAlpha(150);
        strokeWeight(line_width * 2);
      }
      fill(button_l_c);
      rect(mn, button_y, button_width, button_height, button_round);
      fill(0);
      textSize(text_size * 0.8);
      strokeWeight(line_width * 0.5);
      text("starte", mn + button_width * 0.5, button_y + button_height * 0.5 - text_size * 0.25);
      text("Audio", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.75);
      pop()

      renderLevel();
      break;

    case 1:
      inp_auto.show();

      push();
      if (stopwatch_r_running)
        button_l_c = color(button_l_c2);

      button_r_c = color(button_r_c3);
      fill(button_r_c);
      rect(width * 0.5 + mn, button_y, button_width, button_height, button_round);

      if (button_l_pressed) {
        button_l_c.setAlpha(150);
        strokeWeight(line_width * 2);
      }
      fill(button_l_c);
      rect(mn, button_y, button_width, button_height, button_round);
      fill(0);
      strokeWeight(line_width * 0.5);
      if (!stopwatch_l_running && !stopwatch_r_running) {
        textSize(text_size * 0.5);
        text("- PAUSE -", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 - text_size * 4);
        text("Trage oben", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 - text_size * 2);
        text("die Zeit vom ", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 - text_size);
        text("anderen Gerät ein,", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
        text("oder starte", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size * 1);
        text("die Stoppuhr.", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size * 2);
        text("mit einem", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size * 3);
        text("Geräusch.", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size * 4);
      } else if (stopwatch_r_running) {
        textSize(text_size);
        text("STOPP", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
      }
      pop()

      renderLevel();

      push();
      textSize(text_size * 0.6);
      if (button_m_pressed) {
        strokeWeight(line_width * 1.5);
      }
      button_m_c = color(button_m_c1);
      fill(button_m_c);
      ellipse(width * 0.5, button_y + button_height * 0.5, button_rad * 2);
      fill(0);
      strokeWeight(line_width * 0.5);
      text("auto", width * 0.5, button_y + button_height * 0.5 - text_size * 0.15);
      text("start", width * 0.5, button_y + button_height * 0.5 + text_size * 0.55);
      pop();
      break;

    case 2:
      push();
      inp_auto.hide();

      push();
      fill(button_r_c);
      rect(width * 0.5 + mn, button_y, button_width, button_height, button_round);

      if (button_l_pressed) {
        button_l_c.setAlpha(150);
        strokeWeight(line_width * 2);
      }
      fill(button_l_c);
      rect(mn, button_y, button_width, button_height, button_round);
      fill(0);
      textSize(text_size);
      strokeWeight(line_width * 0.5);
      if (!stopwatch_l_running && !stopwatch_r_running)
        text("START", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
      else if (stopwatch_l_running)
        text("STOPP", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
      pop()

      renderLevel();

      push();
      textSize(text_size * 0.6);
      if (button_m_pressed) {
        strokeWeight(line_width * 1.5);
      }
      button_m_c = color(button_m_c2);
      fill(button_m_c);
      ellipse(width * 0.5, button_y + button_height * 0.5, button_rad * 2);
      fill(0);
      strokeWeight(line_width * 0.5);
      text("auto", width * 0.5, button_y + button_height * 0.5 - text_size * 0.15);
      text("stopp", width * 0.5, button_y + button_height * 0.5 + text_size * 0.55);
      pop();
      break;

    case 3:
      inp_auto.show();

      if (auto_pause)
        button_l_c = color(button_l_c1);
      else
        button_l_c = color(button_l_c2);

      push();
      fill(button_r_c);
      rect(width * 0.5 + mn, button_y, button_width, button_height, button_round);

      if (button_l_pressed) {
        button_l_c.setAlpha(150);
        strokeWeight(line_width * 2);
      }
      fill(button_l_c);
      rect(mn, button_y, button_width, button_height, button_round);
      fill(0);
      textSize(text_size * 0.5);
      strokeWeight(line_width * 0.5);
      if (auto_pause) {
        text("- PAUSE -", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 - text_size * 3);
        text("Trage oben", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 - text_size);
        text("die Zeit vom ", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
        text("anderen Gerät ein,", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size);
        text("oder aktiviere", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size * 2);
        text("die Messung.", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size * 3);
      } else {
        text("automatische", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 - text_size);
        text("Messung", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
        text("deaktivieren", mn + button_width * 0.5, button_y + button_height * 0.5 + text_size * 0.25 + text_size);
      }
      pop()

      renderLevel();

      push();
      if (button_m_pressed) {
        strokeWeight(line_width * 1.5);
      }
      button_m_c = color(button_m_c3);
      fill(button_m_c);
      ellipse(width * 0.5, button_y + button_height * 0.5, button_rad * 2);
      fill(0);
      strokeWeight(line_width * 0.5);
      text("auto", width * 0.5, button_y + button_height * 0.5 + text_size * 0.25);
      pop();
      break;
  }
  pop();
}


function renderLevel() {
  let threshold_color = color(0, 0, 120);
  let vol = mic.getLevel() * 25;
  let rad;

  push();
  switch (mode) {
    case 0:

      break;

    case 1:
      if (!stopwatch_r_running && vol_low == true && vol > threshold) {
        start_time = millis();
        stopwatch_r_running = true;
        vol_low = false;
      }

      rad = constrain(map(vol, 0, 1, 0, max_rad), 0, max_rad);
      push();
      fill(220, 220, 0);
      ellipse((width + button_width) * 0.5 + mn, button_y + button_height * 0.5, rad);
      noFill();
      break;

    case 2:
      if (stopwatch_r_running && vol_low == true && vol > threshold) {
        stopwatch_r_running = false;
        end_time_r = end_time_r - 0.03;
        if (end_time_r < 0)
          end_time_r = 0;
        vol_low = false;
      }

      rad = constrain(map(vol, 0, 1, 0, max_rad), 0, max_rad);
      push();
      fill(220, 220, 0);
      ellipse((width + button_width) * 0.5 + mn, button_y + button_height * 0.5, rad);
      noFill();
      break;

    case 3:
      if (!auto_pause && stopwatch_r_running &&
        vol_low == true && vol > threshold) {
        stopwatch_r_running = false;
        vol_low = false;
      }

      if (!auto_pause && !stopwatch_l_running &&
        vol_low == true && vol > threshold) {
        start_time = millis();
        stopwatch_r_running = true;
        vol_low = false;
      }
      rad = constrain(map(vol, 0, 1, 0, max_rad), 0, max_rad);
      push();
      fill(220, 220, 0);
      ellipse((width + button_width) * 0.5 + mn, button_y + button_height * 0.5, rad);
      noFill();
      break;
  }

  if (vol < threshold)
    vol_low = true;

  stroke(threshold_color);
  if (threshold_selected)
    strokeWeight(line_width * 3);
  else
    strokeWeight(line_width * 2);
  ellipse((width + button_width) * 0.5 + mn, button_y + button_height * 0.5, map(threshold, 0, 1, 0, max_rad));
  pop();
}


function changeThreshold() {
  if (!stopwatch_l_running && !stopwatch_r_running) {
    let distance_temp = distSquared(mouseX, mouseY, width * 0.5 + mn + button_width * 0.5, button_y + button_height * 0.5);
    let threshold_temp = map(threshold, 0, 1, 0, max_rad) * 0.5;
    let threshold_temp_min = (threshold_temp - mn) * (threshold_temp - mn);
    let threshold_temp_max = (threshold_temp + mn) * (threshold_temp + mn);

    if (distance_temp < threshold_temp_max && distance_temp > threshold_temp_min)
      threshold_selected = true;
  }

}


function switchMode() {
  stopwatch_l_running = false;
  stopwatch_r_running = false;
  start_time = 0;
  end_time_l = 0;
  end_time_r = 0;
  auto_pause = true;

  mode++;
  if (mode > 3)
    mode = 1;
}


function mousePressed() {
  checkButtons();
  changeThreshold();

  return false;
}


function mouseDragged() {
  if (threshold_selected) {
    threshold = dist(mouseX, mouseY, (width + button_width) * 0.5 + mn, button_y + button_height * 0.5);
    threshold = constrain(map(threshold, 0, max_rad * 0.5, 0, 1), 0.07, 0.97);
  }

}


function touchStarted() {
  checkButtons();

  if (getAudioContext().state !== 'running') {
    getAudioContext().resume();
  }

  changeThreshold();

  return false;
}


function mouseReleased() {
  button_l_pressed = false;
  button_m_pressed = false;
  threshold_selected = false;
}


function touchMoved() {
  if (mouseY < button_y + button_height + mn) {
    if (threshold_selected) {
      threshold = dist(mouseX, mouseY, (width + button_width) * 0.5 + mn, button_y + button_height * 0.5);
      threshold = constrain(map(threshold, 0, max_rad * 0.5, 0, 1), 0.07, 0.97);
    }
    return false;
  }
}


function myInputEvent() {
  inp_dist = this.value();
}


function myInputEventAuto() {
  end_time_l = this.value();
}


function windowResized() {
  resizeCanvas(windowWidth, windowHeight)
  calculateLayout();
}


function distSquared(x1, y1, x2, y2) {
  let dx = x2 - x1;
  let dy = y2 - y1;
  return dx * dx + dy * dy;
}